unit uGTypes;

interface

uses OMEGA, uVar;

type TDirection=( tLeft, tRight, tUp, tDown );

     TFieldElement=record
      Count   : byte;      // 
      Opened  : boolean;
      EOpened : boolean;
      Flag    : boolean;
      NBng    : byte;      //  
      NBColor : integer;   // 
      Time    : integer;
     end;

     TPlayer=object
     public
      Pos           : TPointI;
      Moving        : boolean;
      MoveDirection : TDirection;
      FaceDirection : TDirection;
      MaxTime       : integer;
      Timer         : integer;

      procedure Init;
     protected
      Frame      : byte;   //  
      MovingTime : byte;   //  
     end;

     TField=object
     public
      Cells  : array of array of TFieldElement;
      Weigth,
      Height : integer;

      Map_X, Map_Y : integer;
      Camera       : PCamera2D;
      RoboS        : TPlayer;
      EndLevel,
      Complete,
      Success      : boolean;
      LevelScore   : integer;

      procedure LoadField( patch: string ); // 
      procedure Draw;                      // )
      procedure Process;                  //  
      procedure MP_LButton;              //    . 
      procedure MP_RButton;             //     
      procedure KP( dir: TDirection ); //    
      procedure ActivateCamera;
      procedure DiactivateCamera;
      procedure LevelComplete;
      procedure GameOver;
     protected
      Time    : integer;
      SecTime : byte;

      OpeningCell : boolean;
      LastOpened  : TPointI;

      ExplCell   : boolean;
      Expling    : boolean;
      ExplFrame  : byte;
      ExplCAlpha : byte;
      LastExpl   : TPointI;
      MExplCells : array of TPointI;
      Count_MC   : byte;

      Focus_X, Focus_Y : byte;
      FocusAlpha       : byte;
      Focused          : boolean;

      procedure RefreshNbng;
      procedure OpenEmptyCells( pos: TPointI );
     end;

var Sprites: array of PTexture;
    Count_S: integer;

procedure LoadSprites( patch: string );

implementation

uses uBass;

procedure LoadSprites( patch: string );
var DF: textfile;
    i: integer;
    str: string;
begin
// 
AssignFile( DF, patch );
Reset( DF );
Readln( DF, i );
Count_S:=i;
SetLength( Sprites, Count_S );
for i:=0 to Count_S-1 do begin
 Readln( DF, str );
 Sprites[i]:=Tex_LoadFromFile( PChar(str), -1, TEX_DEFAULT_2D );
end;
CloseFile(DF);
//    
Tex_SetFramesSize( Sprites[10], 64, 64 );
end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure TField.LoadField( patch: string );
var DF:textfile;
    i1, i2, t1, t2: byte;
    value: byte;
begin
Robos.Init;
SetLength( Cells, 0, 0 );

// 
AssignFile( DF, patch );
Reset( DF );
Readln( DF, i1, i2);
Weigth:=i1; Height:=i2;
SetLength( Cells, Weigth, Height );
For i2:=Height-1 downto 0 do begin
 For i1:=0 to Weigth-1 do begin
  Read( DF, value );
  With Cells[i1, i2] do begin
   Count:=value;
   if (Count=9) or (Count=10) then Opened:=true else Opened:=false;
   EOpened:=false;
   NBng:=0;
  end;
 end;
 Readln( DF );
end;
Readln( DF, Robos.Pos.x, Robos.Pos.y );
Readln( DF, Robos.MaxTime );
Readln( DF, i2 );
if i2<>0 then
 For i1:=1 to i2 do begin
  Readln( DF, t1, t2 );
  Cells[t1, t2].Opened:=true;
 end;
CloseFile( DF );

Robos.Timer:=Robos.MaxTime;
SecTime:=0;
Count_MC:=0;
SetLength( MExplCells, Count_MC );
ExplCell := false;  Expling     := false;
Focused  := false;  OpeningCell := false;
EndLevel := false;  Complete    := false;
RefreshNbng;
Camera:=Cam2D_Create;
end;

procedure TField.Draw;
var i1, i2: byte;
    ti: single;
    tb: boolean;
begin

if Focused then begin
 SSprite_Draw( Sprites[5], Map_X+Focus_X*64, Map_Y-(Focus_Y+1)*64, 64, 64 );
end;

For i1:=0 to Weigth-1 do
 For i2:=0 to Height-1 do begin
  tb:=(ABS( i1-Robos.Pos.x )>2) or (ABS( i2-Robos.Pos.y )>2);
  case Cells[i1, i2].Count of

   0:begin
      with Cells[i1, i2] do begin
       if Opened then begin
        // 
        SSprite_Draw(Sprites[1], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64);
        // ,  
        if (NBng>0) and ((not ((Robos.Pos.x=i1) and (Robos.Pos.y=i2))) xor (((Robos.Pos.x=i1) and (Robos.Pos.y=i2)) and Robos.Moving))
         then Text_Draw(GUIFont, Map_X+i1*64+24, Map_Y-(i2+1)*64+20, U_IntToStr( NBng ), 1.0, 0, 255, NBColor);
       end else
        //      
        if tb then SSprite_Draw( Sprites[2], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64 )
         else SSprite_Draw( Sprites[0], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64 );
        if Flag then SSprite_Draw( Sprites[12], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64 );
      end;
     end;

   1:begin
      with Cells[i1, i2] do begin
       if Opened then SSprite_Draw( Sprites[3], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64 )
        else if tb then SSprite_Draw( Sprites[2], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64 )
         else SSprite_Draw( Sprites[0], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64 );
        if Flag then SSprite_Draw( Sprites[12], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64 );
      end;
     end;

   2:SSprite_Draw( Sprites[4], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64 );
   9:SSprite_Draw(Sprites[7], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64);
   10:SSprite_Draw(Sprites[8], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64);
   11:SSprite_Draw(Sprites[6], Map_X+i1*64, Map_Y-(i2+1)*64, 64, 64);
  end;
end;

//  ,   
For i1:=0 to Weigth-1 do
 For i2:=0 to Height-1 do
  with Cells[i1, i2] do begin
  if Time>0 then begin
   ti:=(128+128*(255-Time)/255)/128;
   SSprite_Draw( Sprites[0], Map_X+i1*64-ti*32+32, Map_Y-(i2+1)*64-ti*32+32, ti*64, ti*64, Time );
  end;
end;

with Robos do begin
 if Robos.Moving then
  Case MoveDirection of
   tUp:SSprite_Draw( Sprites[9], Map_X+Pos.x*64, Map_Y-Pos.y*64-2*MovingTime, 64, 64 );
   tDown:SSprite_Draw( Sprites[9], Map_X+Pos.x*64, Map_Y-Pos.y*64-128+2*MovingTime, 64, 64 );
   tLeft:SSprite_Draw( Sprites[9], Map_X+Pos.x*64+64-2*MovingTime, Map_Y-Pos.y*64-64, 64, 64 );
   tRight:SSprite_Draw( Sprites[9], Map_X+Pos.x*64-64+2*MovingTime, Map_Y-Pos.y*64-64, 64, 64 );
  end
 else SSprite_Draw( Sprites[9], Map_X+Robos.Pos.x*64, Map_Y-Robos.Pos.y*64-64, 64, 64 );
end;

if Expling then begin
 ti:=255*(80-ExplCAlpha)/80;
 if Count_MC=0 then begin
  SSprite_Draw( Sprites[11], Map_X+LastExpl.x*64+32-ExplCAlpha*2,
                Map_Y-LastExpl.y*64-32-ExplCAlpha*2, ExplCAlpha*4, ExplCAlpha*4, round( ti ) );
  if ExplFrame<47 then ASprite_Draw( Sprites[10], Map_X+LastExpl.x*64-32, Map_Y-LastExpl.y*64-96, 128, 128, (ExplFrame+1) div 3 );
 end else For i1:=0 to Count_MC-1 do
  with MExplCells[ i1 ] do begin
   SSprite_Draw( Sprites[11], Map_X+X*64+32-ExplCAlpha*2,
                 Map_Y-Y*64-32-ExplCAlpha*2, ExplCAlpha*4, ExplCALpha*4, round( ti ) );
   if ExplFrame<47 then ASprite_Draw( Sprites[10], Map_X+X*64-32, Map_Y-Y*64-96, 128, 128, (ExplFrame+1) div 3 );
  end;
end;

end;

procedure TField.Process;
var i, i1, i2: integer;
begin

if (Cells[Robos.Pos.x, Robos.Pos.y].Count=10) and (not EndLevel) then LevelComplete;

i1:=(Mouse_X-Map_X+round( Camera.X )) div 64;
i2:=(-Mouse_Y+Map_Y-round( Camera.Y )) div 64;

if ((i1>=0) and (i1<=(Weigth-1))) and ((i2>=0) and (i2<=(Height-1))) then begin
  Focus_X:=i1;
  Focus_Y:=i2;
  Focused:=true;
  FocusAlpha:=0;
 end else Focused:=false;

if ((Cells[Focus_X, Focus_Y].Count=11) or (Cells[Focus_X, Focus_Y].Count=10) or (Cells[Focus_X, Focus_Y].Count=2)
 or (Cells[Focus_X, Focus_Y].Count=9)) or (Cells[Focus_X, Focus_Y].Opened)  then Focused:=false;

if (ABS( Focus_X-Robos.Pos.x )>2) or (ABS( Focus_Y-Robos.Pos.y )>2) then Focused:=false;

if Expling then begin
 if ExplCAlpha=80 then begin
  Expling:=false;
  Count_MC:=0;
  SetLength( MExplCells, Count_MC );
 end;
 if ExplFrame<47 then Inc(ExplFrame);
 Inc( ExplCAlpha );
end;

if OpeningCell then begin
 Focused:=false;
 Dec( Cells[LastOpened.x, LastOpened.y].Time, 25 );
 if Cells[LastOpened.x, LastOpened.y].Time=5 then begin
  Cells[LastOpened.x, LastOpened.y].Time:=0;
  OpeningCell:=false;

  if ExplCell then begin
   Cells[LastExpl.x, LastExpl.y].Count:=2;
   Cells[LastExpl.x, LastExpl.y].Flag:=false;
   RefreshNbng;
   ExplCell:=false; Expling:=true;
   ExplFrame:=0;    ExplCAlpha:=1;
   if EnableSound then Play_Sound( Expl_Sound_1 );     ///////
  end;

  if (Cells[LastOpened.x, LastOpened.y].Count=0) and (Cells[LastOpened.x, LastOpened.y].NBng=0)
   and not Cells[LastOpened.x, LastOpened.y].EOpened then begin
    OpenEmptyCells( LastOpened );
    For i1:=0 to Weigth-1 do
     For i2:=0 to Height-1 do Cells[ i1, i2 ].EOpened:=false;
   end;

 end;
end;

if Expling then Focused:=false;
if Robos.Moving then Focused:=false;

if SecTime<40 then Inc( SecTime ) else begin
 SecTime:=0;
 Dec( Robos.Timer );
end;
if (Robos.Timer<=0) and (not Expling) then GameOver;

if Mouse_Press( M_LButton ) then MP_LButton;
if Mouse_Press( M_RButton ) then MP_RButton;
if Key_Press( K_UP ) or Key_Down( K_UP ) then KP( tUp );
if Key_Press( K_DOWN ) or Key_Down( K_DOWN ) then KP( tDown );
if Key_Press( K_LEFT ) or Key_Down( K_LEFT ) then KP( tLeft );
if Key_Press( K_RIGHT ) or Key_Down( K_RIGHT ) then KP( tRight );

With Robos do begin
 if Moving then
  case MoveDirection of
   tUp:begin
        Camera.y:=Camera.Y-2;
        Inc( MovingTime );
   end;
   tDown:begin
          Camera.y:=Camera.Y+2;
          Inc( MovingTime );
   end;
   tLeft:begin
          Camera.x:=Camera.x-2;
          Inc( MovingTime );
   end;
   tRight:begin
          Camera.x:=Camera.x+2;
          Inc( MovingTime );
   end
  end;
 if Moving and (MovingTime=32) then begin
  Moving:=false;
  if Cells[Pos.x, Pos.y].NBng>1 then begin
   i:=0;
   Count_MC:=0;

   if Cells[Pos.x-1, Pos.y+1].Count=1 then begin
    Inc( i );         Inc( Count_MC );
    SetLength( MExplCells, Count_MC );
    MExplCells[ Count_MC-1 ]:=GetPointI( Pos.x-1, Pos.y+1 );
   end;

    if Cells[Pos.x, Pos.y+1].Count=1 then begin
     Inc( i );        Inc( Count_MC );
     SetLength( MExplCells, Count_MC );
     MExplCells[ Count_MC-1 ]:=GetPointI( Pos.x, Pos.y+1 );
    end;

    if Cells[Pos.x+1, Pos.y+1].Count=1 then begin
     Inc( i );        Inc( Count_MC );
     SetLength( MExplCells, Count_MC );
     MExplCells[ Count_MC-1 ]:=GetPointI( Pos.x+1, Pos.y+1 );
    end;

    if Cells[Pos.x-1, Pos.y].Count=1 then begin
     Inc( i );        Inc( Count_MC );
     SetLength( MExplCells, Count_MC );
     MExplCells[ Count_MC-1 ]:=GetPointI( Pos.x-1, Pos.y );
    end;

    if Cells[Pos.x+1, Pos.y].Count=1 then begin
     Inc( i );        Inc( Count_MC );
     SetLength( MExplCells, Count_MC );
     MExplCells[ Count_MC-1 ]:=GetPointI( Pos.x+1, Pos.y );
    end;

    if Cells[Pos.x-1, Pos.y-1].Count=1 then begin
     Inc( i );        Inc( Count_MC );
     SetLength( MExplCells, Count_MC );
     MExplCells[ Count_MC-1 ]:=GetPointI( Pos.x-1, Pos.y-1 );
    end;

    if Cells[Pos.x, Pos.y-1].Count=1 then begin
     Inc( i );        Inc( Count_MC );
     SetLength( MExplCells, Count_MC );
     MExplCells[ Count_MC-1 ]:=GetPointI( Pos.x, Pos.y-1 );
    end;

    if Cells[Pos.x+1, Pos.y-1].Count=1 then begin
     Inc( i );        Inc( Count_MC );
     SetLength( MExplCells, Count_MC );
     MExplCells[ Count_MC-1 ]:=GetPointI( Pos.x+1, Pos.y-1 );
    end;
    Expling:=true;
    ExplFrame:=0;
    ExplCAlpha:=1;
    if EnableSound then Play_Sound( Expl_Sound_2 ); //////
    For i1:=0 to Count_MC-1 do
     with MExplCells[i1] do begin
      Cells[x, y].Count:=2;
      Cells[x, y].Flag:=false;
     end;
    RefreshNbng;
    Timer:=Timer-15*i;
    if Timer<=0 then GameOver;
   end;
 end;

end;

If (EndLevel) and not ((Robos.Moving) or (Expling) or (ExplCell)) then Complete:=true;

end;

procedure TField.MP_LButton;
begin
if Focused then
 if (not Cells[Focus_X, Focus_Y].Opened) and (not Cells[Focus_X, Focus_Y].Flag) then begin

  LastOpened:=GetPointI( Focus_X, Focus_Y );
  Cells[Focus_X, Focus_Y].Time:=255;
  OpeningCell:=true;
  if EnableSound then Play_Sound( Cell_Open_Sound );    ///////

  with Cells[Focus_X, Focus_Y] do begin
   Opened:=true;
   Expling:=false;

   if Count=1 then begin
    LastExpl:=GetPointI( Focus_X, Focus_Y);
    ExplCell:=true;
    Robos.Timer:=Robos.Timer-15;
    if Robos.Timer<=0 then GameOver;
   end;

  end;

 end;
end;

procedure TField.MP_RButton;
begin

if Focused then
 if not Cells[Focus_X, Focus_Y].Flag then Cells[Focus_X, Focus_Y].Flag:=true
  else Cells[Focus_X, Focus_Y].Flag:=false;

end;


procedure TField.KP(dir: TDirection);
begin

if not Robos.Moving then
 case dir of
  tUp:begin
       With Robos do begin
        if ((Pos.y+1)<Height) and (not OpeningCell) and (not Expling) and (Cells[Pos.x, Pos.y+1].Count<>2) then
         if (Cells[Pos.x, Pos.y+1].Opened) and (Cells[Pos.x, Pos.y].Count<11) then begin
          Inc( Pos.y );
          MoveDirection:=dir;
          Moving:=true;
          MovingTime:=0;
         end;
       end;
      end;
  tDown:begin
         With Robos do begin
         if ((Pos.y-1)>-1) and (not OpeningCell) and (not Expling) and (Cells[Pos.x, Pos.y-1].Count<>2) then
          if (Cells[Pos.x, Pos.y-1].Opened) and (Cells[Pos.x, Pos.y].Count<11) then begin
           Dec( Pos.y );
           MoveDirection:=dir;
           Moving:=true;
           MovingTime:=0;
          end;
         end;
        end;
  tLeft:begin
         With Robos do begin
          if ((Pos.x-1)>-1) and (not OpeningCell) and (not Expling) and (Cells[Pos.x-1, Pos.y].Count<>2) then
           if (Cells[Pos.x-1, Pos.y].Opened) and (Cells[Pos.x-1, Pos.y].Count<11) then begin
            Dec( Pos.x );
            MoveDirection:=dir;
            Moving:=true;
            MovingTime:=0;
           end;
         end;
        end;
  tRight:begin
          With Robos do begin
           if ((Pos.x+1)<Weigth) and (not OpeningCell) and (not Expling) and (Cells[Pos.x+1, Pos.y].Count<>2) then
            if (Cells[Pos.x+1, Pos.y].Opened) and (Cells[Pos.x, Pos.y].Count<11) then begin
             Inc( Pos.x );
             MoveDirection:=dir;
             Moving:=true;
             MovingTime:=0;
            end;
          end;
         end;
 end;

end;

procedure TField.ActivateCamera;
begin
Cam2D_Set( Camera );
end;

procedure TField.DiactivateCamera;
begin
Cam2D_Free( Camera );
end;

procedure TField.GameOver;
begin
EndLevel:=true;
Success:=false;
Complete:=false;
LevelScore:=-20;
end;

procedure TField.LevelComplete;
var i1, i2: integer;
begin
if EnableSound then Play_Sound( Level_Complete_S ); ////
EndLevel:=true;
Success:=true;
Complete:=false;

LevelScore:=RoboS.Timer;
For i1:=0 to Weigth-1 do
 For i2:=0 to Height-1 do begin
  if Cells[ i1, i2 ].Flag and (Cells[ i1, i2 ].Count=1) then INC( LevelScore, 15 );
  if Cells[ i1, i2 ].Flag and (Cells[ i1, i2 ].Count<>1) then DEC( LevelScore, 5 );
 end;
  
end;

procedure TField.RefreshNbng;
var i, i1, i2: byte;
begin

For i1:=0 to Weigth-1 do
 For i2:=0 to Height-1 do
  if (Cells[i1, i2].Count=0) then begin
   i:=0;
   if Cells[i1-1, i2+1].Count=1 then Inc(i);
   if Cells[i1, i2+1].Count=1 then Inc(i);
   if Cells[i1+1, i2+1].Count=1 then Inc(i);
   if Cells[i1-1, i2].Count=1 then Inc(i);
   if Cells[i1+1, i2].Count=1 then Inc(i);
   if Cells[i1-1, i2-1].Count=1 then Inc(i);
   if Cells[i1, i2-1].Count=1 then Inc(i);
   if Cells[i1+1, i2-1].Count=1 then Inc(i);
   with Cells[i1, i2] do begin
    NBng:=i;
    case NBng of
     1:NBColor:=$666DFF;
     2:NBColor:=$1E881E;
     3:NBColor:=$A81000;
     4:NBColor:=$0000B4;
     5:NBColor:=$ffffff;
     6:NBColor:=$ffffff;
     7:NBColor:=$ffffff;
     8:NBColor:=$ffffff;
    end;
   end;
  end;

end;

procedure TField.OpenEmptyCells(pos: TPointI);
begin

Cells[ pos.x, pos.y ].Opened:=true;
Cells[ pos.x, pos.y ].EOpened:=true;
Cells[ pos.x, pos.y ].Flag:=false;

if (Cells[ pos.x+1, pos.y ].Count=0) and (Cells[ pos.x+1, pos.y ].NBng=0) and not Cells[ pos.x+1, pos.y ].EOpened
 then OpenEmptyCells( GetPointI( pos.x+1, pos.y ) )
 else begin
  Cells[ pos.x+1, pos.y ].Opened:=true;
  Cells[ pos.x+1, pos.y ].EOpened:=true;
  Cells[ pos.x+1, pos.y ].Flag:=false;
 end;

if (Cells[ pos.x-1, pos.y ].Count=0) and (Cells[ pos.x-1, pos.y ].NBng=0) and not Cells[ pos.x-1, pos.y ].EOpened
 then OpenEmptyCells( GetPointI( pos.x-1, pos.y ) )
 else begin
  Cells[ pos.x-1, pos.y ].Opened:=true;
  Cells[ pos.x-1, pos.y ].EOpened:=true;
  Cells[ pos.x-1, pos.y ].Flag:=false;
 end;

if (Cells[ pos.x+1, pos.y+1 ].Count=0) and (Cells[ pos.x+1, pos.y+1 ].NBng=0) and not Cells[ pos.x+1, pos.y+1 ].EOpened
 then OpenEmptyCells( GetPointI( pos.x+1, pos.y+1 ) )
 else begin
  Cells[ pos.x+1, pos.y+1 ].Opened:=true;
  Cells[ pos.x+1, pos.y+1 ].EOpened:=true;
  Cells[ pos.x+1, pos.y+1 ].Flag:=false;
 end;

if (Cells[ pos.x, pos.y+1 ].Count=0) and (Cells[ pos.x, pos.y+1 ].NBng=0) and not Cells[ pos.x, pos.y+1 ].EOpened
 then OpenEmptyCells( GetPointI( pos.x, pos.y+1 ) )
 else begin
  Cells[ pos.x, pos.y+1 ].Opened:=true;
  Cells[ pos.x, pos.y+1 ].EOpened:=true;
  Cells[ pos.x, pos.y+1 ].Flag:=false;
 end;

if (Cells[ pos.x-1, pos.y+1 ].Count=0) and (Cells[ pos.x-1, pos.y+1 ].NBng=0) and not Cells[ pos.x-1, pos.y+1 ].EOpened
 then OpenEmptyCells( GetPointI( pos.x-1, pos.y+1 ) )
 else begin
  Cells[ pos.x-1, pos.y+1 ].Opened:=true;
  Cells[ pos.x-1, pos.y+1 ].EOpened:=true;
  Cells[ pos.x, pos.y+1 ].Flag:=false;
 end;

if (Cells[ pos.x+1, pos.y-1 ].Count=0) and (Cells[ pos.x+1, pos.y-1 ].NBng=0) and not Cells[ pos.x+1, pos.y-1 ].EOpened
 then OpenEmptyCells( GetPointI( pos.x+1, pos.y-1 ) )
 else begin
  Cells[ pos.x+1, pos.y-1 ].Opened:=true;
  Cells[ pos.x+1, pos.y-1 ].EOpened:=true;
  Cells[ pos.x+1, pos.y-1 ].Flag:=false;
 end;

if (Cells[ pos.x, pos.y-1 ].Count=0) and (Cells[ pos.x, pos.y-1 ].NBng=0) and not Cells[ pos.x, pos.y-1 ].EOpened
 then OpenEmptyCells( GetPointI( pos.x, pos.y-1 ) )
 else begin
  Cells[ pos.x, pos.y-1 ].Opened:=true;
  Cells[ pos.x, pos.y-1 ].EOpened:=true;
  Cells[ pos.x, pos.y-1 ].Flag:=false;
end;

if (Cells[ pos.x-1, pos.y-1 ].Count=0) and (Cells[ pos.x-1, pos.y-1 ].NBng=0) and not Cells[ pos.x-1, pos.y-1 ].EOpened
 then OpenEmptyCells( GetPointI( pos.x-1, pos.y-1 ) )
 else begin
  Cells[ pos.x-1, pos.y-1 ].Opened:=true;
  Cells[ pos.x-1, pos.y-1 ].EOpened:=true;
  Cells[ pos.x-1, pos.y-1 ].Flag:=false;  
 end;

end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

procedure TPlayer.Init;
begin
 FaceDirection:=tUp;
 MoveDirection:=tUp;
 Moving:=false;
 Timer:=0;
 Pos:=GetPointI( 0, 0 );
end;

end.
