unit Gui;

interface

uses omega, Utils;

const btnCount  = 4;
      btnWidth  = 96;
      btnHeigth = 32;
      btnRange  = 16;

//------------------------------------------------------------------------
type TGlButton = class
       bRect:      TGLRect;  // 
       bCaption:   String;   //
       bStatus:    boolean;  //(    )
       bProc:      procedure;//   
       bAlfa:      integer;  //
       bEnable:    boolean;  //
       TextColor:  integer;  // 
       constructor Create(x,y,w,h,a,txtColor: integer; text: string);
       procedure   Process;
       procedure   Click;
       procedure   Draw;
       destructor  Destroy; override;
     end;

type TGlCheckBox = class
       Rect: TGLRect;
       Status: integer;//0 -  1 - 
       Caption: string;
       Enable: boolean;
       TextColor: integer;
       Proc: procedure;
       constructor Create(x,y,w,h,s,txtColor: integer; text: string);
       procedure Process;
       procedure Draw;
       destructor Destroy; override;
     end;

//  
type TGLEdit = class
       Rect: TGLRect;
       Enable: boolean;
       Caption: string;
       CaptionPlace: integer; //   0 -  1  2 - 
       Status: boolean; // 0 -  1 -  
       Proc: procedure;
       SizeCur: integer;
       PosCur: integer;
       TextEdit: string;
       TimerCur: integer;
       MaxLen: integer;
       textColor: integer;
       constructor Create(x,y,w,h,CP,ML,txtColor: integer; Cap, Text: string; s: boolean);
       procedure Process;
       procedure Draw;
       destructor Destroy; override;
     end;

procedure BtnCreate;
procedure BtnProcess;
procedure BtnDraw;
procedure BtnDestroy;
procedure OptionCreate;
procedure OptionOpen;
procedure OptionDraw;
procedure OptionProcess;
procedure ChooseEasy;
procedure ChooseMedium;
procedure ChooseHard;
procedure ClickApply;
procedure ClickDefault;
procedure ClickCancel;
procedure Play;
procedure RecordCreate;
procedure RecordDraw;
procedure RecordProcess;
procedure RecordOpen;
procedure RecordAdd;
procedure RecordClose;
procedure RecordOk;
procedure RecordContinue;

var btnGame, btnSetting, btnRecord, btnExit: TGlButton;
    btnDefaultOption, btnApplyOption, btnCancelOption: TGlButton;
    chbMusic, chbSound, chbEasy, chbMedium, chbHard, chbColor: TGlCheckBox;
    edtName: TGLEdit;
    btnRecordContinue: TGlButton;
    texBtn, texChb1, texChb2, texTile1, texTile2,
    texFlag, texMine, texSmile1, texSmile2,
    texEdit, texCursor: PTexture;

implementation

uses Main;

{ TGlButton }

//------------------------------------------------------------------------------
procedure TGlButton.Click;
begin
  if Mouse_Click(M_LBUTTON) then
  begin
    if @bProc <> nil then
    begin
      bProc;
    end;
  end;
end;

//------------------------------------------------------------------------------
constructor TGlButton.Create(x, y, w, h, a,txtColor: integer; text: string);
begin
  bRect:=GLRect(x,y,w,h);
  bCaption:= text;
  bStatus:= False;
  bAlfa:= a;
  bEnable:= True;
  bProc:= nil;
  TextColor:= txtColor;
end;

//------------------------------------------------------------------------------
destructor TGlButton.Destroy;
begin
  inherited;
end;

//------------------------------------------------------------------------------
procedure TGlButton.Draw;
var w,h: integer;
begin
  if bEnable = false then exit;

  w:= round(Text_GetWidth(Font,pchar(bCaption)));
  h:= 15;
  if bStatus = false then
  begin
    if DrawSprite = true then
    begin
      SSprite_Draw(texBtn,bRect.x,bRect.y,btnWidth,btnHeigth);
    end
    else
    begin
      Pr2D_Rect(bRect.x,bRect.y,bRect.w,bRect.h,$111111,255,PR_FILL);
      Pr2D_LineWidth(1);
      Pr2D_Rect(bRect.x,bRect.y,bRect.w+1,bRect.h,$CCCCCC);
    end;
    Text_Draw(Font, bRect.x + bRect.w div 2 - w div 2,
               bRect.y + bRect.h div 2 - h div 2,PChar(bCaption),1,0,bAlfa,textColor);
  end
  else
  begin
    if DrawSprite = true then
    begin
      SSprite_Draw(texBtn,bRect.x,bRect.y,btnWidth,btnHeigth,155);
    end
    else
    begin
      Pr2D_Rect(bRect.x,bRect.y,bRect.w,bRect.h,$333333,150,PR_FILL);
      Pr2D_LineWidth(1);
      Pr2D_Rect(bRect.x,bRect.y,bRect.w+1,bRect.h,$CCCCCC);
    end;
    Text_Draw(Font, bRect.x + bRect.w div 2 - w div 2,
               bRect.y + bRect.h div 2 - h div 2,PChar(bCaption),1,0,bAlfa,textColor);
  end;
end;

//------------------------------------------------------------------------------
procedure TGlButton.Process;
begin
  if bEnable = false then exit;

  if PointInRect(Mouse_X,Mouse_Y,bRect) = true then
  begin
    bStatus:= true;
    Click;
  end
  else
    bStatus:= false;
end;

{ TGlCheckBox }

constructor TGlCheckBox.Create(x, y, w, h, s,txtColor: integer; text: string);
begin
  Rect:= GLRect(x,y,w,h);
  Status:= s;
  Caption:= text;
  Enable:= false;
  TextColor:= txtColor;
  Proc:= nil;
end;

destructor TGlCheckBox.Destroy;
begin
  inherited;
end;

procedure TGlCheckBox.Draw;
var h: integer;
begin
  if Status = 0 then
  begin
    if DrawSprite = true then
    begin
      SSprite_Draw(texChb2,Rect.x,Rect.y,24,24);
    end
    else
    begin
      Pr2D_Rect(Rect.x,Rect.y,Rect.w,Rect.h,$111111,255,PR_FILL);
      Pr2D_LineWidth(1);
      Pr2D_Line(Rect.x,Rect.y,Rect.x+Rect.w,Rect.y+Rect.h,$CCCCCC);
      Pr2D_Line(Rect.x+Rect.w,Rect.y,Rect.x,Rect.y+Rect.h,$CCCCCC);
      Pr2D_Rect(Rect.x,Rect.y,Rect.w+1,Rect.h,$CCCCCC);
    end;
  end
  else
  begin
    if DrawSprite = true then
    begin
      SSprite_Draw(texChb1,Rect.x,Rect.y,24,24);
    end
    else
    begin
      Pr2D_Rect(Rect.x,Rect.y,Rect.w,Rect.h,$333333,255,PR_FILL);
      Pr2D_LineWidth(1);
      Pr2D_Rect(Rect.x,Rect.y,Rect.w+1,Rect.h,$CCCCCC);
    end;
  end;
  h:= 15;
  Text_Draw(Font,Rect.x+Rect.w+10,
             Rect.y + Rect.h div 2 - h div 2,PChar(Caption),1,0,200,textColor);
end;

procedure TGlCheckBox.Process;
begin
 if (PointInRect(Mouse_X,Mouse_Y,Rect) = true) and
    (Mouse_Click(M_LBUTTON) = true) then
 begin
   if Status = 0 then
     Status:= 1
   else
     Status:= 0;
   if @Proc <> nil then
   begin
     Proc;
   end;
 end;
end;

{ TGLEdit }

constructor TGLEdit.Create(x,y,w,h,CP,ML,txtColor: integer; Cap, Text: string; s: boolean);
begin
  Rect:= GLRect(x,y,w,h);
  Enable:= false;
  Caption:= Cap;
  CaptionPlace:= CP;
  Status:= s;
  Proc:= nil;
  SizeCur:= 15;
  PosCur:= 0;
  TextEdit:= Text;
  TimerCur:=0;
  MaxLen:= ML;
  textColor:= txtColor;
end;

destructor TGLEdit.Destroy;
begin
  inherited;
end;

procedure TGLEdit.Draw;
var xC,yC: integer;
    tmpText: string;
    i, firstNmb: integer;
    flag: boolean;
begin
  if Enable = false then
    exit;
  if DrawSprite = true then
  begin
    SSprite_Draw(texEdit,Rect.x,Rect.y,96,24);
  end
  else
  begin
    Pr2D_Rect(Rect.x,Rect.y,Rect.w,Rect.h,$111111,255,PR_FILL);
    Pr2D_LineWidth(1);
    Pr2D_Rect(Rect.x,Rect.y,Rect.w+1,Rect.h,$CCCCCC);
  end;
  if Text_GetWidth(Font,pChar(TextEdit))>Rect.w-3 then
  begin
    flag:= true;
    firstNmb:= 2;
    while flag do
    begin
      tmpText:= '';
      for i:= firstNmb to length(TextEdit) do
      begin
        tmpText:= tmpText+TextEdit[i];
      end;
      if Text_GetWidth(Font,pChar(tmpText)) <= Rect.w-3 then
        flag:= false;
      inc(firstNmb,1);
    end;
    if (TimerCur < 45) and (Status = true) then
    begin
      xC:= Rect.x+3+Round(Text_GetWidth(Font,pChar(tmpText)))+1;
      yC:= Rect.y+(Rect.h div 2)-(SizeCur div 2);
      Pr2D_LineWidth(2);
      Pr2D_Rect(xC,yC,1,SizeCur,textColor);
    end;
    Text_Draw(Font,Rect.x+3,(Rect.y+(Rect.h div 2)-(SizeCur div 2)),
              PChar(tmpText),1,0,255,textColor);
  end
  else
  begin
    if (TimerCur < 45) and (Status = true) then
    begin
      xC:= Rect.x+3+Round(Text_GetWidth(Font,pChar(TextEdit)))+1;
      yC:= Rect.y+(Rect.h div 2)-(SizeCur div 2);
      Pr2D_LineWidth(2);
      Pr2D_Rect(xC,yC,1,SizeCur,textColor);
    end;
    Text_Draw(Font,Rect.x+3,(Rect.y+(Rect.h div 2)-(SizeCur div 2)),
               PChar(TextEdit),1,0,255,textColor);
  end;
  case CaptionPlace of
    0:begin
        Text_Draw(Font,Rect.x-Text_GetWidth(Font,pChar(Caption))-2,
        (Rect.y+(Rect.h div 2)-(SizeCur div 2)),PChar(Caption),1,0,255,textColor);
      end;
    1:begin
        Text_Draw(Font,Rect.x,
        Rect.y-Text_GetWidth(Font,pChar(Caption))-2,PChar(Caption),1,0,255,textColor);
      end;
    2:begin
        Text_Draw(Font,Rect.x+Rect.w+2,
        (Rect.y+(Rect.h div 2)-(SizeCur div 2)),PChar(Caption),1,0,255,textColor);
      end;
  end;
end;

procedure TGLEdit.Process;
begin
  if Enable = false then
    exit;
  if PointInRect(Mouse_X,Mouse_Y,Rect) = true then
  begin
    if Mouse_Click(M_LBUTTON) then
    begin
      Key_MaxReadText(MaxLen);
      Key_BeginReadText(PChar(TextEdit));
      Status:= true;
    end;
  end
  else
  begin
    if ((Mouse_Click(M_LBUTTON) = true) or (Key_Down(K_ENTER) = true)) and (Status = true) then
    begin
      //Caption := Key_EndReadText;
      if @Proc <> nil then
      begin
        Proc;
      end;
      Status:= false;
    end;
  end;
  if Status then
  begin
    if TimerCur > 90 then
      TimerCur := 0
    else
      inc(TimerCur,1);
    TextEdit:= Key_EndReadText;
  end;
end;

procedure BtnCreate;
begin
  btnGame:= TGlButton.Create(OE_GetOptions.wnd_Width div 2 -
                             (btnCount*btnWidth+btnRange*(btnCount-1)) div 2,
                             4,btnWidth,btnHeigth,255,$FFFFFF,'');
  btnSetting:= TGlButton.Create((OE_GetOptions.wnd_Width div 2 -
                                (btnCount*btnWidth+btnRange*(btnCount-1)) div 2)+(btnRange+btnWidth),
                                4,btnWidth,btnHeigth,255,$FFFFFF,'');
  btnRecord:= TGlButton.Create((OE_GetOptions.wnd_Width div 2 -
                               (btnCount*btnWidth+btnRange*(btnCount-1)) div 2)+(btnRange+btnWidth)*2,
                               4,btnWidth,btnHeigth,255,$FFFFFF,'');
  btnExit:= TGlButton.Create((OE_GetOptions.wnd_Width div 2 -
                             (btnCount*btnWidth+btnRange*(btnCount-1)) div 2)+(btnRange+btnWidth)*3,
                             4,btnWidth,btnHeigth,255,$FFFFFF,'');
  btnExit.bProc:= @OE_Quit;
  btnSetting.bProc:= @OptionOpen;
  btnGame.bProc:= @Play;
  btnRecord.bProc:= @RecordOpen;
  btnGame.bEnable:= true;
  btnSetting.bEnable:= true;
  btnRecord.bEnable:= true;
  btnExit.bEnable:= true;
end;

procedure BtnProcess;
begin
  if FlagAction = FlagGame then
  begin
    btnGame.Process;
    btnSetting.Process;
    btnRecord.Process;
  end;
  btnExit.Process;
end;

procedure BtnDraw;
begin
  btnGame.Draw;
  btnSetting.Draw;
  btnRecord.Draw;
  btnExit.Draw;
end;

procedure BtnDestroy;
begin
  btnGame.Free;
  btnSetting.Free;
  btnRecord.Free;
  btnExit.Free;
end;

procedure OptionCreate;
begin
  MusicOn:= defaultMusic;
  SoundOn:= defaultSound;
  GameLevel:= defaultLevel;
  DrawSprite:= defaultColor;
  btnDefaultOption:= TGlButton.Create(32,198,btnWidth,btnHeigth,255,$FFFFFF,' ');
  btnApplyOption:= TGlButton.Create(32+btnWidth+btnRange,198,btnWidth,btnHeigth,255,$FFFFFF,'');
  btnCancelOption:= TGlButton.Create(32+(btnWidth+btnRange)*2,198,btnWidth,btnHeigth,255,$FFFFFF,'');
  btnCancelOption.bProc:= @ClickCancel;
  btnDefaultOption.bProc:= @ClickDefault;
  btnApplyOption.bProc:= @ClickApply;
  btnDefaultOption.bEnable:= true;
  btnApplyOption.bEnable:= true;
  btnCancelOption.bEnable:= true;
  chbMusic:= TGlCheckBox.Create(32,70,24,24,1,$FFFFFF,'');
  chbSound:= TGlCheckBox.Create(32,102,24,24,0,$FFFFFF,'');
  chbEasy:= TGlCheckBox.Create(138,70,24,24,0,$FFFFFF,'˸');
  chbMedium:= TGlCheckBox.Create(138,102,24,24,1,$FFFFFF,'');
  chbHard:= TGlCheckBox.Create(138,134,24,24,1,$FFFFFF,'');
  chbColor:= TGlCheckBox.Create(32,134,24,24,1,$FFFFFF,'/ ');
  chbEasy.Proc:= @ChooseEasy;
  chbMedium.Proc:= @ChooseMedium;
  chbHard.Proc:= @ChooseHard;
  chbMusic.Enable:= true;
  chbSound.Enable:= true;
  chbEasy.Enable:= true;
  chbMedium.Enable:= true;
  chbHard.Enable:= true;
  chbColor.Enable:= true;
end;

procedure OptionOpen;
begin
  FlagAction:= FlagSetting;
end;

procedure OptionDraw;
begin
  btnDefaultOption.Draw;
  btnApplyOption.Draw;
  btnCancelOption.Draw;
  chbMusic.Draw;
  chbSound.Draw;
  chbEasy.Draw;
  chbMedium.Draw;
  chbHard.Draw;
  chbColor.Draw;
end;

procedure OptionProcess;
begin
  btnDefaultOption.Process;
  btnApplyOption.Process;
  btnCancelOption.Process;
  chbMusic.Process;
  chbSound.Process;
  chbEasy.Process;
  chbMedium.Process;
  chbHard.Process;
  chbColor.Process;
end;

procedure ChooseEasy;
begin
  chbEasy.Status:= 0;
  chbMedium.Status:= 1;
  chbHard.Status:= 1;
end;

procedure ChooseMedium;
begin
  chbEasy.Status:= 1;
  chbMedium.Status:= 0;
  chbHard.Status:= 1;
end;

procedure ChooseHard;
begin
  chbEasy.Status:= 1;
  chbMedium.Status:= 1;
  chbHard.Status:= 0;
end;

procedure ClickApply;
var tmpLevel: integer;
begin
  MusicOn:= chbMusic.Status;
  SoundOn:= chbSound.Status;
  if MusicOn = 0 then
    Media_Play('data\music.mid',true)
  else
    Media_Stop;
  tmpLevel:= GameLevel;
  if chbEasy.Status = 0 then
    GameLevel:= 1;
  if chbMedium.Status = 0 then
    GameLevel:= 2;
  if chbHard.Status = 0 then
    GameLevel:= 3;
  if chbColor.Status = 1 then
    DrawSprite:= true
  else
    DrawSprite:= false;
  if tmpLevel <> GameLevel then
  begin
    case GameLevel of
      1: PlayGame(10,10,10);
      2: PlayGame(20,15,70);
      3: PlayGame(40,25,200);
    end;
  end
  else
    FlagAction:= FlagGame;
end;

procedure ClickDefault;
begin
  ChooseEasy;
  chbMusic.Status:= defaultMusic;
  chbSound.Status:= defaultSound;
  chbColor.Status:= 1;
end;

procedure ClickCancel;
begin
  chbMusic.Status:= MusicOn;
  chbSound.Status:= SoundOn;
  if DrawSprite = true then
    chbColor.Status:= 1
  else
    chbColor.Status:= 0;
  case GameLevel of
    1:ChooseEasy;
    2:ChooseMedium;
    3:ChooseHard;
  end;
  if StatusGame = 0 then
    FlagAction:= FlagGame
  else
    Play;
end;

procedure Play;
begin
  case GameLevel of
    1: PlayGame(10,10,10);
    2: PlayGame(20,15,70);
    3: PlayGame(40,25,200);
  end;
end;

procedure RecordCreate;
begin
  edtName:= TGLEdit.Create(64,256,96,24,0,16,$FFFFFF,'','Player',false);
  edtName.Proc:= @RecordOk;
  btnRecordContinue:= TGlButton.Create(256,256,90,32,255,$FFFFFF,'');
  btnRecordContinue.bProc:= @RecordContinue;
  btnRecordContinue.bEnable:= false;
end;

procedure RecordDraw;
var numStr: integer;
    tmpStr: string;
begin
  if StatusGame = 1 then
    Text_Draw(Font,64,230,PChar('!!!'));
  Text_Draw(Font,32,70,PChar(''));
  Text_Draw(Font,160,70,PChar(''));
  for numStr:= 1 to 5 do
  begin
    tmpStr:= RecordTable[GameLevel,numStr].Name;
    Text_Draw(Font,16,70+24*numStr,PChar(U_IntToStr(numStr)+')'));
    Text_Draw(Font,32,70+24*numStr,PChar(tmpStr));
    Text_Draw(Font,160,70+24*numStr,U_IntToStr(RecordTable[GameLevel,numStr].Time));
  end;
  edtName.Draw;
  btnRecordContinue.Draw;
end;

procedure RecordProcess;
begin
  edtName.Process;
  btnRecordContinue.Process;
end;

procedure RecordOpen;
begin
  FlagAction:= FlagRecord;
  btnRecordContinue.bEnable:= true;
end;

procedure RecordAdd;
begin
  FlagAction:= FlagRecord;
  edtName.Status:= True;
  edtName.Enable:= True;
end;

procedure RecordClose;
begin
  if StartGame = true then
    FlagAction:= FlagGame
  else
    Play;
end;

procedure RecordOk;
var numStr, i: integer;
begin
  edtName.Enable:= False;
  numStr:= 5;
  for i:= 1 to 5 do
  begin
  if RecordTable[GameLevel,i].Time >= GameTime then
  begin
    numStr:= i;
    Break;
  end;
  end;
  for i:= 5 downto numStr+1 do
  begin
    RecordTable[GameLevel,i].Name:= RecordTable[GameLevel,i-1].Name;
    RecordTable[GameLevel,i].Time:= RecordTable[GameLevel,i-1].Time;
  end;
  RecordTable[GameLevel,numStr].Name:= edtName.TextEdit;
  RecordTable[GameLevel,numStr].Time:= GameTime;
  edtName.TextEdit:= 'Player';
  SaveRecordTable;
  btnRecordContinue.bEnable:= true;
end;

procedure RecordContinue;
begin
  if StatusGame = 0 then
    FlagAction:= FlagGame
  else
    Play;
  btnRecordContinue.bEnable:= false;
end;

end.
