unit Main;

interface

uses omega;

const FlagGame     = 0;
      FlagSetting  = 1;
      FlagRecord   = 2;
      defaultMusic = 1;
      defaultSound = 0;
      defaultLevel = 1;
      defaultColor = true;

var TimeTimer   : POTimer;
    UpdateTimer : POTimer;
    GameTime    : integer;
    Font        : PFont2D;
    PoleWidth   : integer;
    PoleHeight  : integer;
    MineCount   : integer;
    FlagCount   : integer;
    OpenPole    : integer;
    FlagOnMine  : integer;
    StartGame   : boolean;
    PauseGame   : boolean;
    StatusGame  : integer;//0-  1-  2-
    FlagAction  : integer;// 0-  1-  2-
    MusicOn     : integer;
    SoundOn     : integer;
    GameLevel   : integer;
    SoundExpl   : PSound;
    SoundEmpty  : PSound;
    IdExpl      : integer;
    idEmpty     : integer;
    DrawSprite  : boolean;

procedure MainInit;
procedure MainDraw;
procedure MainProcess;
procedure MainQuit;
procedure PlayGame(Width,Height,Mine: integer);
procedure EndGameProcess;
procedure WinGame;
procedure LooseGame;

implementation

uses Utils, gui;

procedure MainInit;
begin
  Snd_Init;
  Randomize;
  Media_Play('data\music.mid',true);
  Media_Stop;
  texBtn:= Tex_LoadFromFile('data\Button.tga',-1,TEXTYPE_TGA);
  texChb1:= Tex_LoadFromFile('data\check1.tga',-1,TEXTYPE_TGA);
  texChb2:= Tex_LoadFromFile('data\check2.tga',-1,TEXTYPE_TGA);
  texTile1:= Tex_LoadFromFile('data\tile1.tga',-1,TEXTYPE_TGA);
  texTile2:= Tex_LoadFromFile('data\tile2.tga',-1,TEXTYPE_TGA);
  texFlag:= Tex_LoadFromFile('data\Flag.tga',-1,TEXTYPE_TGA);
  texMine:= Tex_LoadFromFile('data\Mina.tga',-1,TEXTYPE_TGA);
  texSmile1:= Tex_LoadFromFile('data\smile1.tga',-1,TEXTYPE_TGA);
  texSmile2:= Tex_LoadFromFile('data\smile2.tga',-1,TEXTYPE_TGA);
  texEdit:= Tex_LoadFromFile('data\Edit.tga',-1,TEXTYPE_TGA);
  texCursor:= Tex_LoadFromFile('data\cursor.tga',-1,TEXTYPE_TGA);
  Font := Text_LoadFontFromFile('data\font.tga', 'data\mainfont.ofi',
                                -1, TEX_DEFAULT_2D or TEX_FILTER_NEAREST);
  SoundExpl:= Snd_LoadFromFile('data\expl.wav', 1);
  SoundEmpty:= Snd_LoadFromFile('data\empty.wav', 1);
  BtnCreate;
  OptionCreate;
  RecordCreate;
  LoadRecordTable;
  PlayGame(10,10,10);
  FlagAction:= FlagGame;
end;

procedure MainDraw;
begin
  Pr2D_Rect(0,0,OE_GetOptions.wnd_Width,OE_GetOptions.wnd_Height,$777777,255,PR_FILL);
  BtnDraw;
  case FlagAction of
    FlagGame:
    begin
      PoleDraw;
      SmileDraw;
      Text_Draw(Font,OE_GetOptions.wnd_Width/2+32,48,PChar(': '+U_IntToStr(GameTime)));
      Text_Draw(Font,OE_GetOptions.wnd_Width div 2-(Length(Pole)*24+(Length(Pole)*2)+2) div 2,
                70+26*PoleHeight+16,PChar(': '+U_IntToStr(FlagCount)+'/'+U_IntToStr(MineCount)));
      if StatusGame = 2 then
        Text_Draw(Font,OE_GetOptions.wnd_Width div 2-(Length(Pole)*24+(Length(Pole)*2)+2) div 2,
                  48,PChar('...'));
    end;
    FlagSetting:
    begin
      OptionDraw;
    end;
    FlagRecord:
    begin
      RecordDraw;
    end;
  end;
  Text_Draw(Font,OE_GetOptions.wnd_Width-64,OE_GetOptions.wnd_Height-32,PChar('FPS: '+U_IntToStr(OE_GetFPS)));
  CursorDraw;
end;

procedure MainProcess;
begin
  case FlagAction of
    FlagGame:
    begin
      PauseGame:= false;
      if StatusGame <> 2 then
      begin
        EndGameProcess;
        PoleProcess;
      end;
    end;
    FlagSetting:
    begin
      PauseGame:= true;
      OptionProcess;
    end;
    FlagRecord:
    begin
      PauseGame:= true;
      RecordProcess;
    end;
  end;
  BtnProcess;
end;

procedure MainQuit;
begin
  Text_FreeFont(Font);
  SetLength(Pole,0,0);
  BtnDestroy;
end;

procedure PlayGame(Width,Height,Mine: integer);
begin
  PoleWidth:= Width;
  PoleHeight:= Height;
  case GameLevel of
    1: OE_ChangeScreenOptions(btnCount*btnWidth+btnRange*(btnCount-1)+32, 374, 32, 0, False, False);
    2,3: OE_ChangeScreenOptions(PoleWidth*26+32, PoleHeight*26+128, 32, 0, False, False);
  end;
  OE_SetWndPosition(50,100);
  MineCount:= Mine;
  GameTime:= 0;
  StatusGame:= 0;
  StartGame:= False;
  FlagCount:= MineCount;
  OpenPole:= 0;
  FlagOnMine:= 0;
  SetLength(Pole,0,0);
  PoleInit(PoleWidth,PoleHeight);
  FlagAction:= FlagGame;
end;

procedure EndGameProcess;
begin
  if (OpenPole = PoleWidth*PoleHeight-MineCount) or
     (FlagOnMine = MineCount) then
    WinGame;
end;

procedure WinGame;
begin
  StatusGame:=1;
  RecordAdd;
end;

procedure LooseGame;
var x,y: integer;
begin
  for x:= 0 to PoleWidth-1 do
    for y:= 0 to PoleHeight-1 do
    begin
      Pole[x,y].isVisible:= true;
    end;
  StatusGame:= 2;
end;

end.
