unit NFAFont;
{
  by Hex[man]
  Date: 27.04.2009

  Simple Font Engine
}
interface
uses Windows, OpenGL, sNail;


type
  TNFASymbol = record
    l, t, w, h:single;
    size:byte;
  end;

  TNFAData = record
    symbHeight:byte;
    symbols:array[32..255] of TNFASymbol;
  end;
  
  PNFAFont = ^TNFAFont;
  TNFAFont = object
  private
    data:TNFAData;
    tex_font:cardinal;
  public
    procedure Load(const FileName:string);
    procedure SetTexture(const Texture:cardinal);
    procedure Render(x, y:single; Str:string; Step:shortint = -2);

    function StrWidth(const Str:string; Step:shortint = -2):word;
    function StrHeight:byte;
    function SymbolWidth(const symb:char):byte;
    procedure Free;
  end;

implementation

{ TNFAFont }

procedure TNFAFont.Load(const FileName: string);
var
  F:file of byte;
begin
  ZeroMemory(@data,sizeof(data));
  AssignFile(F, FileName);
  Reset(F);
  BlockRead(F,data,FileSize(F));
  CloseFile(F);
end;

procedure TNFAFont.Render(x, y: single; Str: string; Step:shortint = -2);
var
  i:word;
  w2,h2:single;
  xpos,ypos,symbsize:word;
begin
  gl_TexEnable(tex_font);
  h2 := (data.symbHeight / 2);
  xpos := 0;
  ypos := 0;

  for i := 1 to Length(Str) do
  begin
    if Str[i] = #13 then
    begin
      xpos := 0;
      inc(ypos, data.symbHeight);
      continue;
    end;

    symbsize:=data.symbols[ord(Str[i])].size;
    w2:=(symbsize / 2);

    glPushMatrix;
    glTranslatef(x+w2+xpos, y+h2+ypos, 0);

    glBegin(GL_QUADS);
      with data.symbols[ord(Str[i])] do
      begin
        glTexCoord2f(l, t); glVertex2f(-w2, -h2);
        glTexCoord2f(w, t); glVertex2f(w2, -h2);
        glTexCoord2f(w, h); glVertex2f(w2, h2);
        glTexCoord2f(l, h); glVertex2f(-w2, h2);
      end;
    glEnd;
    glPopMatrix;

    inc(xpos, symbsize + Step);
  end;

  gl_TexDisable;
end;

procedure TNFAFont.SetTexture(const Texture: cardinal);
begin
  tex_font:=Texture;
end;

procedure TNFAFont.Free;
begin

end;

function TNFAFont.StrWidth(const Str: string; Step:shortint = -2): word;
var
  i, totalX: word;
begin
  totalX:=0;
  for i:=1 to Length(Str) do
    inc(totalX, data.symbols[ord(Str[i])].size + Step);
  Result:=totalX;
end;

function TNFAFont.StrHeight: byte;
begin
  Result:=data.symbHeight;
end;

function TNFAFont.SymbolWidth(const symb:char):byte;
begin
  Result:=data.symbols[ord(symb)].size;
end;

end.
