unit uGeneral;
interface

uses Windows, sNail, OpenGL, NFAFont, uParticles, Math;

const
  MAX_CELLS_X = 30;
  MAX_CELLS_Y = 16;
  CELL_SIZE = 24;

  FRAME_BORDER = 5;
  FRAME_ALPHA = 100;

type
  TGameMode = (gmBeginner, gmIntermediate, gmExpert, gmLoser);

  PMinesField = ^TMinesField;
  TCell = object
  private
    idX, idY : word;
    parent: PMinesField;

    x, y: single;
    value, alpha: byte;
    function GetTimerSprite: shortint;
  public
    opened,
    selected,
    click,
    flag: boolean;
    timer: single;

    procedure SetPos(cx, cy: single);
    procedure Update;
    procedure Render;
  end;

  TGameButton = object
  private
    x, y: word;
    w, h: word;
    caption: string;
    color: integer;
    click: boolean;
  public
    procedure Init(dx, dy: word; c: string);
    procedure Update;
    procedure Render;
  end;

  TTopData = record
    beginner,
    intermediate,
    expert,
    loser: word;
  end;

  TMinesField = object
  private
    posX, posY, width, height: word;
    cellsX, cellsY, maxMines: byte;
    cells: array[0..MAX_CELLS_X - 1, 0..MAX_CELLS_Y - 1] of TCell;
    opened: word;

    gameOver, isWin: boolean;
    time: single;
    flagCount: shortint;

    btReset, btMenu: TGameButton;
    function GetMinesAround(cellx, celly: byte): byte;
    procedure GenMineField(cellX, cellY: byte);
    procedure ClearZeroCells(x, y: byte);
    procedure SetFlag(var f: boolean);
    procedure Open(x, y: word);
    procedure RenderHUD;
    procedure WriteBestTime;
    procedure ReadBestTime;
    procedure SetBestTime(t: word);
  public
    gameMode: TGameMode;
    bestTime: TTopData;
    procedure Init;
    procedure Clear(gm: TGameMode);
    procedure ResetMe;
    procedure Update;
    procedure Render;
    procedure Free;
  end;

  TBgLine = object
    up: boolean;
    x, speed: single;
    size: byte;

    procedure Init;
    procedure RandParam;
    procedure Update;
    procedure Render;
  end;

  procedure sndPlay(id: integer);
var
  minesField: TMinesField;
  font: TNFAFont;
  texFont, texTimer, texC4: cardinal;
  texCellsOpen, texCellsClose, texFlag: cardinal;
  lines: array[0..9] of TBgLine;
  particles: TPartEngine;

  soundEnable : boolean;
  sndBombActive, sndNewFiled, sndExplosion, sndClick, sndOpen, sndError: integer;
  
implementation
uses uMenu;

procedure sndPlay(id: integer);
begin
  if not soundEnable then exit;
  snd_Play(id);
end;

{ TCell }

procedure TCell.SetPos(cx, cy: single);
begin
  x := cx;
  y := cy;
end;

procedure TCell.Render;
begin
  gl_SetColor($ffffff, alpha);
  if opened then
  begin
    gl_TexEnable(texCellsOpen);
    glPushMatrix;
      gl_RenderSprite(x, y, CELL_SIZE, CELL_SIZE, 4, 4, value + GetTimerSprite, 0);
    glPopMatrix;
    gl_TexDisable;
  end else
  begin
    gl_TexEnable(texCellsClose);
    glPushMatrix;
      gl_RenderSprite(x, y, CELL_SIZE, CELL_SIZE, 2, 2, byte(selected) + byte(click), 0);
    glPopMatrix;
    gl_TexDisable;

    if flag then
    begin
      gl_TexEnable(texFlag);
      glPushMatrix;
        gl_RenderTex(x, y, CELL_SIZE, CELL_SIZE, 0);
      glPopMatrix;
      gl_TexDisable;
    end;
  end;
  
end;

procedure TCell.Update;
begin
  if (opened) or (parent.gameOver) then
  begin
    if value = 9 then
    begin
      if timer > 2.9 then
      begin
        particles.Add(PART_BOOM, x + (CELL_SIZE shr 1), y + (CELL_SIZE shr 1), $ffffff);
        sndPlay(sndExplosion);
        timer := 0;
        value := 14;
      end;
      timer := timer + 0.001 * snail_GetTimer;
    end;
    exit;
  end;

  selected := PtInRect(Rect(Round(x), Round(y), Round(x) + CELL_SIZE, Round(y) + CELL_SIZE), inp_GetMousePos);

  if timer = 0 then
  begin
    if (alpha < 250) then
    begin
      alpha := alpha + 5;
    end else
    begin

      if selected then
      begin
        if inp_MouseUp(MOUSE_BTN_RIGHT) then
        begin
          parent.SetFlag(flag);
        end;

        if not flag then
        begin
          if inp_MouseUp(MOUSE_BTN_LEFT) and click then
          begin
            opened := true;
            parent.Open(idX, idY);
          end;
          click := inp_MousePress(MOUSE_BTN_LEFT);
        end;

      end else
      click := false;
    end;
  end else timer := timer + 1.0;
end;

function TCell.GetTimerSprite: shortint;
begin
  if timer < 0 then
    result := 0
  else
    result := round(timer - 0.5);
end;

{ TMinesField }

procedure TMinesField.Clear(gm: TGameMode);
var
  i, j: byte;
begin
  case gm of
    gmBeginner:
    begin
      cellsX := 9;
      cellsY := 9;
      maxMines := 10;
    end;
    gmIntermediate:
    begin
      cellsX := 16;
      cellsY := 16;
      maxMines := 40;
    end;
    gmExpert:
    begin
      cellsX := MAX_CELLS_X;
      cellsY := MAX_CELLS_Y;
      maxMines := 99;
    end;
    gmLoser:
    begin
      cellsX := MAX_CELLS_X;
      cellsY := MAX_CELLS_Y;
      maxMines := 10;
    end;
  end;

  gameMode := gm;
  opened := 0;
  gameOver := false;
  isWin := false;
  flagCount := maxMines;
  time := -2.8;

  posX := (gl_GetResX shr 1) - ((cellsX * CELL_SIZE) shr 1);
  posY := (gl_GetResY shr 1) - ((cellsY * CELL_SIZE) shr 1);

  width := (CELL_SIZE * cellsX);
  height := (CELL_SIZE * cellsY); 

  btReset.Init(posX + width - 82, posY + height + 20, 'Reset');
  btMenu.Init(posX + width - 82, posY + height + 45, 'Menu');

  ReadBestTime;
  FillChar(cells, sizeof(cells), 0);
  for i := 0 to cellsX - 1 do
    for j := 0 to cellsY - 1 do
      with cells[i, j] do
      begin
        idX := i;
        idY := j;
        alpha := 0;
        timer := -round(Random(MAX_CELLS_X * MAX_CELLS_Y) / 24);
        parent := @self;
        SetPos(posX + (i * CELL_SIZE), posY + (j * CELL_SIZE));
      end;
      
  snd_StopAll;
  sndPlay(sndNewFiled);
end;

procedure TMinesField.GenMineField(cellX, cellY: byte);
var
  i, j, x, y: byte;
  empty: boolean;
begin
  for i := 0 to cellsX - 1 do
    for j := 0 to cellsY - 1 do
      cells[i, j].value := 0;

  for i := 0 to maxMines - 1 do
  begin
    empty := true;
    repeat
      x := Random(cellsX);
      y := Random(cellsY);
      if (cells[x, y].value = 0) and (x <> cellY) and (y <> cellY) then
      begin
        cells[x, y].value := 9;
        empty := false;
      end;
    until not empty;
  end;

  for i := 0 to cellsX - 1 do
    for j := 0 to cellsY - 1 do
    begin
      if cells[i, j].value in [1..8, 9] then
        continue;

      cells[i, j].value := GetMinesAround(i, j);
    end;
end;

procedure TMinesField.ClearZeroCells(x, y: byte);
var
  i, j: shortint;
begin
  cells[x, y].opened := true;
  inc(opened);
  if (cells[x, y].value = 0) then
    for i := -1 to 1 do
        for j := -1 to 1 do
          if ((i <> 0) or (j <> 0)) and (x + i >= 0) and (x + i < cellsX) and
             (y + j >= 0) and (y + j < cellsY) and (not cells[x + i, y + j].opened) and
             (not cells[x + i, y + j].flag)
          then
            ClearZeroCells(x + i, y + j);

end;

function TMinesField.GetMinesAround(cellx, celly: byte): byte;
var
  i, j: shortint;
  mines: byte;
begin
  mines := 0;
  for i := cellx - 1 to cellx + 1 do
    for j := celly - 1 to celly + 1 do
    begin
      if (i < 0) or (j < 0) or (i > cellsX - 1) or (j > cellsY - 1) or
         (cells[i, j].value in [1..8]) then continue;

      if cells[i, j].value = 9 then inc(mines);
    end;
  result := mines;
end;

procedure TMinesField.Init;
begin
  particles := TPartEngine.Create;
  ReadBestTime;
end;

procedure TMinesField.Open(x, y: word);
var
  i, j: byte;
begin
  if opened = 0 then GenMineField(x, y);

  case cells[x, y].value of
    0:
    begin
      ClearZeroCells(x, y);
      sndPlay(sndOpen);
    end;
    1..8:
    begin
      inc(opened);
      sndPlay(sndClick);
    end;
    9:
    begin
      sndPlay(sndBombActive);
      for i := 0 to cellsX - 1 do
        for j := 0 to cellsY - 1 do
        begin
          if ((x = i) and (y = j)) then continue;

            with cells[i, j] do
            begin
              if value = 9 then
              begin
                gameOver := true;
                isWin := false;
                opened := true;
                timer := -random * 3;
              end else
                if flag then
                begin
                  value := 15;
                  opened := true;
                end;
            end;

        end;
    end;
  end;
  
  if (maxMines = (cellsX * cellsY) - opened) then
  begin
    for i := 0 to cellsX - 1 do
      for j := 0 to cellsY - 1 do
        if not cells[i, j].opened then cells[i, j].flag := true;

    gameOver := true;
    isWin := true;
    SetBestTime(round(time));
    WriteBestTime;
  end;
end;

procedure TMinesField.SetFlag(var f: boolean);
begin
  if not f then
  begin
    if flagCount = 0 then
    begin
      if not snd_isPlay(sndError) then sndPlay(sndError);
      exit;
    end;
    dec(flagCount)
  end
  else inc(flagCount);
  f := not f;
end;

procedure TMinesField.RenderHUD;
begin
  gl_SetColor($ffffff, 255);
  gl_TexEnable(texTimer);
  glPushMatrix;
    gl_RenderTex(posX + 7,posY - 45, 32, 32, 0);
  glPopMatrix;
  gl_TexDisable;

  gl_TexEnable(texC4);
  glPushMatrix;
    gl_RenderTex(posX + 100,posY - 45, 32, 32, 0);
  glPopMatrix;
  gl_TexDisable;

  font.Render(posX + 45, posY - 40, Int2Str(round(abs(time))));
  font.Render(posX + 140, posY - 40, Int2Str(flagCount));

  btReset.Render;
  btMenu.Render;
  
  if gameOver then
  begin
    gl_SetColor($ffffff, 255);

    if isWin then
    begin
      font.Render((gl_GetResX shr 1) - (font.StrWidth('You win.') shr 1), 20, 'You win.');
    end else
    begin
      font.Render((gl_GetResX shr 1) - (font.StrWidth('You lose. Try again please.') shr 1), 10, 'You lose. Try again please.');
    end;

  end;
end;

procedure TMinesField.Render;
var
  i, j: byte;
begin
  gl_SetColor($000000, FRAME_ALPHA);
  glPushMatrix;
    glRect(posX - FRAME_BORDER, posY - FRAME_BORDER,
           posX + (CELL_SIZE * cellsX) + FRAME_BORDER, posY + (CELL_SIZE * cellsY) + FRAME_BORDER);
  glPopMatrix;

  glPushMatrix;
    glRect(posX, posY - 50, posX + 190, posY - FRAME_BORDER);
  glPopMatrix;

  glPushMatrix;
    glRect(posX + width - 100, posY + height + FRAME_BORDER, posX + width, posY + height + 80);
  glPopMatrix;
  
  for i := 0 to cellsX - 1 do
    for j := 0 to cellsY - 1 do
      cells[i, j].Render;
 
  particles.Render;
  RenderHUD;
end;

procedure TMinesField.Update;
var
  i, j: integer;
begin
  particles.Update;
  btReset.Update;
  btMenu.Update;
  
  if (inp_KeyUp(VK_RETURN)) or (btReset.click) then ResetMe;
  if (inp_KeyUp(VK_ESCAPE)) or (btMenu.click) then
  begin
    isMenu := true;
    menu.FirstSection;
    snd_StopAll;
  end;

  if not gameOver then
  begin
    time := time + 0.001 * snail_GetTimer;

    if time >= 999 then
    begin
      gameOver := true;
      isWin := false;
    end;
  end;

  if isWin then exit;

  for i := 0 to cellsX - 1 do
    for j := 0 to cellsY - 1 do
      cells[i, j].Update;
end;

procedure TMinesField.Free;
begin
end;

procedure TMinesField.ResetMe;
begin
  Clear(gameMode);
end;

procedure TMinesField.ReadBestTime;
var
  f: file of byte;
begin
  AssignFile(f, 'top.dtn');
  Reset(f);
  BlockRead(f, bestTime, sizeof(TTopData));
  CloseFile(f);
end;

procedure TMinesField.WriteBestTime;
var
  f: file of byte;
begin
  AssignFile(f, 'top.dtn');
  Rewrite(f);
  BlockWrite(f, bestTime, sizeof(TTopData));
  CloseFile(f);
end;

procedure TMinesField.SetBestTime(t: word);
begin       
  with bestTime do
  case gameMode of
    gmBeginner: beginner := min(beginner, t);
    gmIntermediate: intermediate := min(intermediate, t);
    gmExpert: expert := min(expert, t);
    gmLoser: loser := min(loser, t);
  end;
end;

{ TBgLine }

procedure TBgLine.Init;
begin
  x := round(Random(gl_GetResX));
  up := boolean(random(2));
  RandParam;
end;

procedure TBgLine.RandParam();
begin
  size := random(5) + 1;
  speed := trunc(random(59) + 1) * 0.01;
end;

procedure TBgLine.Render;
begin
  glPushMatrix;
  glLineWidth(size);
  gl_SetColor($000000, 15);
  glBegin(GL_LINES);
    glVertex2f(x, 0);
    glVertex2f(x, gl_GetResY);
  glEnd;
  glPopMatrix;
end;

procedure TBgLine.Update;
begin
  if up then
  begin
    x := x - speed;
    if x < -size then
    begin
      up := false;
      RandParam;
    end;
  end else
  begin
    x := x + speed;
    if x > gl_GetResX then
    begin
      up := true;
      RandParam;
    end;
  end;
end;

{ TGameButton }

procedure TGameButton.Init(dx, dy: word; c: string);
begin
  x := dx;
  y := dy;
  caption := c;
  w := font.StrWidth(caption);
  h := font.StrHeight;
end;

procedure TGameButton.Render;
begin
  gl_SetColor(color);
  font.Render(x, y, caption);
end;

procedure TGameButton.Update;
begin
  if PtInRect(Rect(x, y, x + w, y + h), inp_GetMousePos) then
  begin
    color := $ff0000;
    click := inp_MouseUp(MOUSE_BTN_LEFT);
  end else
  begin
    color := $ffffff;
    click := false;
  end;
end;

end.

