unit uMenu;

interface
uses Windows, sNail, OpenGL, NFAFont, uGeneral;

type
  TMenuSection = (msMain, msCredits, msNewGame, msBestTime, msHelp);

  TButton = object
  private
    x, y, w, h: word;
    caption: string;
    iconId: byte;
    click: boolean;
    color: integer;
  public
    procedure Init(dy: word; c: string; icon: byte);
    procedure Update;
    procedure Render;
  end;

  TBtnSect = object
  private
    btn: TButton;
    section: TMenuSection;
  public
    procedure Init(dy: word; c: string; icon: byte; ms: TMenuSection);
    procedure Update;
    procedure Render;
  end;
  
  TBtnMode = object
  private
    btn: TButton;
    gameMode: TGameMode;
  public
    procedure Init(dy: word; c: string; icon: byte; gm: TGameMode);
    procedure Update;
    procedure Render;
  end;

  TMenu = object
  private
    btQuit: TButton;
    btNewGame, btCredits, btTop, btHelp, btBack: TBtnSect;
    btMode1,btMode2, btMode3, btMode4: TBtnMode;
    procedure Title(str: string);
  public
    procedure FirstSection;
    procedure Init;
    procedure Update;
    procedure Render;
  end;

var
  menu: TMenu;
  isMenu: boolean = true;
  texLogo: cardinal;
  msection: TMenuSection;
  
implementation

{ TButton }

procedure TButton.Init(dy: word; c: string; icon: byte);
begin
  y := dy;
  caption := c;
  w := font.StrWidth(caption);
  h := font.StrHeight;
  x := (gl_GetResX shr 1) - (w shr 1);
  color := $ffffff;
  iconId := icon;
end;

procedure TButton.Render;
begin
  gl_SetColor($ffffff);
  gl_TexEnable(texCellsOpen);
  glPushMatrix;
    gl_RenderSprite((gl_GetResX shr 1) - 110, y -2, CELL_SIZE, CELL_SIZE, 4, 4, iconId, 0);
  glPopMatrix;
  gl_TexDisable;

  gl_SetColor(color);
  font.Render(x, y, caption);
end;

procedure TButton.Update;
begin
  if PtInRect(Rect(x, y, x + w, y + h), inp_GetMousePos) then
  begin
    color := $ff0000;
    click := inp_MouseUp(MOUSE_BTN_LEFT);
    if click then sndPlay(sndClick);
  end else
  begin
    color := $ffffff;
  end;
end;

{ TMenu }

procedure TMenu.FirstSection;
begin
  msection := msMain;
end;

procedure TMenu.Init;
begin
  msection := msMain;
  btNewGame.Init(150, 'New Game', 1, msNewGame);
  btTop.Init(200, 'Best Time', 2, msBestTime);
  btHelp.Init(250, 'Help', 3, msHelp);
  btCredits.Init(300, 'Credits', 4, msCredits);
  btQuit.Init(460, 'Quit', 15);
  btBack.Init(460, 'Back', 15, msMain);

  btMode1.Init(150, 'Beginner', 1, gmBeginner);
  btMode2.Init(200, 'Intermediate', 2, gmIntermediate);
  btMode3.Init(250, 'Expert', 3, gmExpert);
  btMode4.Init(300, 'Loser', 4, gmLoser);
end;

procedure TMenu.Render;
begin
  gl_SetColor($ffffff);
  gl_TexEnable(texLogo);
  glPushMatrix;
    gl_RenderTex((gl_GetResX shr 1) - (tex_GetSize(texLogo).X shr 1), 20, tex_GetSize(texLogo).x, tex_GetSize(texLogo).y, 0);
  glPopMatrix;
  gl_TexDisable;

  gl_SetColor($000000, FRAME_ALPHA);
  glPushMatrix;
    glRect(0, 130, gl_GetResX, 500);
  glPopMatrix;

  case msection of
    msMain:
    begin
      btNewGame.Render;
      btTop.Render;
      btHelp.Render;
      btCredits.Render;
      btQuit.Render;
    end;

    msNewGame:
    begin
      title('Choose game mode');
      btMode1.Render;
      btMode2.Render;
      btMode3.Render;
      btMode4.Render;
    end;

    msCredits:
    begin
      title('Credits');
      gl_SetColor($ffffff);
      font.Render(10, 200, 'Detonator for igdc.ru game contest.'+#13+#13+#13+
      'Programmer and game designer: Kirill Hex[man] Khorobrykh'+#13+'3D designer: Maxim Mark Khorobrykh'+#13+#13+
      'Email: hex.man[at]mail.ru');
    end;
    
    msBestTime:
    begin
      title('Best time');
      gl_SetColor($ffffff);
      font.Render(10, 200, 'Beginner : ' + Int2Str(minesField.bestTime.beginner) + ' sec');
      font.Render(10, 250, 'Intermediate : ' + Int2Str(minesField.bestTime.intermediate) + ' sec');
      font.Render(10, 300, 'Expert : ' + Int2Str(minesField.bestTime.expert) + ' sec');
      font.Render(10, 350, 'Loser : ' + Int2Str(minesField.bestTime.loser) + ' sec');
    end;

    msHelp:
    begin
      title('Help');
      gl_SetColor($ffffff);
      font.Render(10, 200, 'Game Mode: '+#13+'Beginner : field 9x9 , 10 mines'+#13+
                           'Intermediate : field 16x16 , 40 mines'+#13+
                           'Expert : field 30x16 , 99 mines'+#13+
                           'Loser : field 30x16 , 10 mines'+#13+#13+
                           'Controls:'+#13+'Left Mouse Button : Open Cell'+#13+
                           'Right Mouse Button : Set flag'+#13+
                           'Enter : Reset game'+#13+
                           'Escape : Back to menu');
    end;
  end;

  if msection <> msMain then btBack.Render;

end;

procedure TMenu.Title(str: string);
begin
  gl_SetColor($d2781e);
  font.Render(20, 150, str);
end;

procedure TMenu.Update;
begin
  case msection of
    msMain:
    begin
      btNewGame.Update;
      btTop.Update;
      btHelp.Update;
      btCredits.Update;
      btQuit.Update;

      if btQuit.click then snail_Stop;
    end;

    msNewGame:
    begin
      btMode1.Update;
      btMode2.Update;
      btMode3.Update;
      btMode4.Update;
    end;
  end;

  btBack.Update;

end;

{ TBtnMode }

procedure TBtnMode.Init(dy: word; c: string; icon: byte; gm: TGameMode);
begin
  btn.Init(dy, c, icon);
  gameMode := gm;
end;

procedure TBtnMode.Render;
begin
  btn.Render;
end;

procedure TBtnMode.Update;
begin
  btn.Update;
  if btn.click then
  begin
    isMenu := false;
    minesField.Clear(gameMode);
    btn.click := false;
  end;
end;

{ TBtnSect }

procedure TBtnSect.Init(dy: word; c: string; icon: byte; ms: TMenuSection);
begin
  btn.Init(dy, c, icon);
  section := ms;
end;

procedure TBtnSect.Render;
begin
  btn.Render;
end;

procedure TBtnSect.Update;
begin
  btn.Update;
  if btn.click then
  begin
    msection := section;
    btn.click := false;
  end;
end;

end.
