unit g_map;

interface
uses
  g_effect;
const
  ws_Open = 1;
  ws_Cast = 2;

  et_Mine   = 1;
  et_Saph   = 2;
  et_RedB   = 3;
  et_BlueB  = 4;
  et_RedP   = 5;
  et_GreenP = 6;
  et_BlueP  = 7;

type
  TFT   = record                              // 
    Ic1, Ic2, N : Byte;                       // , , 
    X, Y, Size  : Single;                     // 
  end;

  TTile = record                              // 
    Open, EFree,                              // , 
    nEnable, Down      : Boolean;             //  , 
    Alpha              : Single;              //   ( )
    Tmr,                                      // 
    MofT, MofTT, MofBT : Byte;                //  : , , 
    FT                 : array[1..4] of TFT;  // 
  end;

  TMap = class                                // 
    // 
    Tile   : array of array of TTile;         //  
    DTile  : TDTile;                          //  
    Effect : TSEffect;                        // 
    W, H   : Word;                            //  
    StX, StY,                                 //  
    Wd2, Hd2 : Single;                        //  
    ChTile_X,
    ChTile_Y : Byte;                          //   

    procedure Init(FileName : string);        // 
    procedure Gen;
    procedure Calc;
    procedure Tile_Open(iX, iY : Integer);
    procedure AddDTile(X, Y : Integer);
    procedure Work(mX, mY : Single;           //    
                     mPress : Boolean;        //  
                     Sost : Byte = 0);        //    
    procedure Update;
    procedure Draw;
  end;
var
  Wimd  : Integer;

implementation
uses
  OpenGL,
  GSound,
  LTe_friend,
  LexUtils,
  Setup,
  g_materials,
  g_size,
  g_opt,
  g_main
  ;
//______________________________________________________________________________
//                    
//______________________________________________________________________________
procedure TMap.Init;
var
  IX, IY : Word;
  F : file;
begin
  AssignFile(F, FileName);
  Reset(F, 1);
  BlockRead(F, W, SizeOf(Word));
  BlockRead(F, H, SizeOf(Word));

  Wd2 := W / 2;
  Hd2 := H / 2;

  //    H*W
  SetLength(Tile, W + 1, H + 1);
  //  
  for IX := 0 to W  do
  for IY := 0 to H  do
  with Tile[IX, IY] do
  begin
    Open    := False;
    EFree   := True;
    Alpha   := 1;
    Down    := False;
    BlockRead(F, nEnable, SizeOf(Boolean));
    Tmr := 0;
  end;
  BlockRead(F, Opt, SizeOf(TOpt));
  CloseFile(F);

  //   
  Opt_2_nill;
  //
  if DTile = nil then
    DTile := TDTile.Create;

  if Effect = nil then
    Effect := TSEffect.Create;

  //  
  Gen;

end;

procedure TMap.Gen;
var
  iX, iY : Integer;

  function PosGen: Boolean;
  begin
    iX := Random(W + 1);
    iY := Random(H + 1);
    with Tile[iX, iY] do
    if  (not nEnable)
    and (EFree)
    and (not Open) then
    begin
      EFree    := False;
      Result   := True;
      MofT     := 1;
      FT[1].X  := 0;
      FT[1].Y  := 0;
    end
    else
      Result := False;
  end;

  procedure ElGen(I, Z : Integer);
  begin
    while I <> 0 do
    if PosGen then
    with Tile[iX, iY].FT[1] do
    begin
      Ic1 := Z;
      Dec(I);
    end;
  end;
var
  PS : Integer;
begin
  for PS := 1 to 7 do
  ElGen(Opt.MofE_ME[PS, 1] - Opt.MofE_ME[PS, 2], PS);
  //  
  Calc;
end;

procedure TMap.Calc; //  
var
  iX, iY, cX, cY, rX, rY, N, M : Integer;
begin
  for IX := 0 to W  do
  for IY := 0 to H  do
  if Tile[IX, IY].EFree then
  begin
    N := 0;
    M := 0;
    for cX := -1 to 1 do
    for cY := -1 to 1 do
    begin
      rX := iX + cX;
      rY := iY + cY;
      if  (rX >= 0)
      and (rX <= W)
      and (rY >= 0)
      and (rY <= H)
      and (not Tile[rX, rY].EFree) then
      case Tile[rX, rY].FT[1].Ic1 of
        1 : Inc(N);
        2 : Inc(M);
      end;
    end;
    Tile[IX, IY].FT[3].N := M;
    Tile[IX, IY].FT[4].N := N;

    if (N > 0) and (M > 0) then
    begin
      Tile[IX, IY].FT[3].X     := - 2;
      Tile[IX, IY].FT[3].Size  := 0.5;
      Tile[IX, IY].FT[4].X     := + 2;
      Tile[IX, IY].FT[4].Size  := 0.5;
    end
    else
    if (N > 0) then
    begin
      Tile[IX, IY].FT[4].X     := 0;
      Tile[IX, IY].FT[4].Size  := 0.8;
    end
    else
    begin
      Tile[IX, IY].FT[3].X     := 0;
      Tile[IX, IY].FT[3].Size  := 0.8;
    end;

  end;
end;
//______________________________________________________________________________
//                     
//______________________________________________________________________________
//  
procedure TMap.Tile_Open;
var
  eX, eS_2T, eS_T : Integer;
begin
  with Tile[iX, iY] do
  if  (not Down)
  and (not nEnable) then
  begin
    Alpha := 0.99;
    Down  := True;
    AddDTile(iX, iY);
    Tmr := 15;
    // :
    PlaySound(sound_flash);
    // :
    eS_T  :=  Round(s_T*1.5);
    eS_2T := eS_T div 3;
    eS_T  := eS_2T * 2;
    for eX := 1 to 5 do
      Expl_Add(lm_Expl, StX + iX * s_T + Random(eS_T) - eS_2T, StY + iY * s_T + Random(eS_T) - eS_2T, s_T/2 + Random(eS_2T), Int2RGB(0), 0)
  end;
end;

//  :
procedure TMap.Work;

  procedure Tile_Cast(X, Y : Integer);
  begin
    playsound(sound_cast);
    if not Tile[X, Y].Down then
    begin
      Tile[X, Y].Alpha := 0.99;
      Wimd  := ws_Open;
      Dec(Game.Force);
    end;
  end;

begin
  ChTile_X := Round((mX - StX )/ s_T);
  ChTile_Y := Round((mY - StY) / s_T);
  if  (ChTile_X <= W)
  and (ChTile_Y <= H) then
  begin
    if mPress then
    case Sost of
      ws_Open : Tile_Open(ChTile_X, ChTile_Y);
      ws_Cast : Tile_Cast(ChTile_X, ChTile_Y);
    end;
  end;
end;

// 
procedure TMap.Update;
var
  IX, IY : Integer;

  procedure nFree_Tile_Open;
  begin
    with Effect.Element[Effect.Add] do
    begin
      Enabled := True;
      pcX     := IX;
      pcY     := IY;
      Alpha   :=  1.2;
      ppAlpha := -0.02;
      Size    := 0.5;
      ppSize  := 0.02;
      Tp      := Tile[IX, IY].FT[1].Ic1;
      case Tp of
        1 : playsound(sound_boom);
        2 : playsound(sound_exp);
        3, 5 : playsound(sound_Red);
        4, 7 : playsound(sound_Blue);
        6 :    playsound(sound_Green);
      end;
    end;
  end;

  procedure Null_Tile_Open;
  var
  cX, cY, rX, rY : Integer;
  begin
    Tile[IX, IY].Open := True;
    with Tile[IX, IY] do
    if (not EFree) then
      nFree_Tile_Open
    else
    if  (FT[4].N = 0)
    and (not nEnable) then
    for cX := -1 to 1 do
    for cY := -1 to 1 do
    begin
      rX := iX + cX;
      rY := iY + cY;
      if  (rX >= 0)
      and (rX <= W)
      and (rY >= 0)
      and (rY <= H)
      and (not Tile[rX, rY].Down)
      and (Tile[rX, rY].EFree)
      and (not Tile[rX, rY].nEnable)
      then
        Tile_Open(rX, rY)
    end;
  end;
begin

  for IX := 0 to W  do
  for IY := 0 to H  do
  with Tile[IX, IY] do
  begin
    if  (not Open)
    and (Alpha < 1)
    and (Alpha > 0.7) then
      App(Alpha, -0.01);

    if Tmr > 0 then
    begin
      if Tmr > 1 then
        Dec(Tmr)
      else
      begin
        Tmr := 0;
        Null_Tile_Open;
      end;
    end;
  end;

  DTile.Work;
  for IX := 0 to Length(Effect.Element) - 1  do
  if Effect.Work(IX) then
  case Effect.Element[IX].Tp of
    1 : Game.Bomb;
    2 : Game.Saph;
    3 : Game.RedBotle;
    4 : Game.BlueBotle;
    5 : begin
          if Game.LiveMax < 7 then
            Inc(Game.LiveMax);
          Game.RedBotle;
        end;
    6 : begin
          if Game.LiveMax < 7 then
            Inc(Game.LiveMax);
          if Game.ForceMax < 7 then  
            Inc(Game.ForceMax);
        end;
    7 : begin
          if Game.ForceMax < 7 then
            Inc(Game.ForceMax);
          Game.BlueBotle;
        end;
  end;
end;

procedure TMap.AddDTile;
begin
  with DTile.Element[DTile.Add] do
  begin
    Enabled :=  True;
    Alpha   :=  1;
    mdX     :=  0;
    mdY     := -0.1;
    pcX     :=  X;
    pcY     :=  Y;
    ppR     :=  0.5 * (Random(5) - 2);
  end;
end;

//______________________________________________________________________________
//                    
//______________________________________________________________________________
procedure TMap.Draw;
var
  IX, IY : Integer;
  
  //  
  procedure Tile_Free_Draw;
  begin
    with Tile[IX, IY].FT[4] do
    if N > 0 then
    begin
      glTranslated(  X * S_TF, 0, 0);
      OpenGL.glColor4f(0, 0, 0, 1);
      Draw2D_ESprite(lm_FontC, N, s_TE * Size);
      glTranslated( - X * S_TF, 0, 0);
    end;
    with Tile[IX, IY].FT[3] do
    if N > 0 then
    begin
      glTranslated(  X * S_TF, 0, 0);
      OpenGL.glColor4f(0, 0, 1, 1);
      Draw2D_ESprite(lm_FontC, N, s_TE * Size);
      glTranslated( - X * S_TF, 0, 0);
    end;
    OpenGL.glColor4f(1, 1, 1, 1);
  end;

  procedure Tile_NotFree_Draw;
  begin
    Draw2D_ESprite(lm_t_el[Tile[IX, IY].FT[1].Ic1], 0, s_TE)
  end;

  procedure Tile_Draw;
  begin
    if  (ChTile_X = IX)
    and (ChTile_Y = IY) then
      Draw2D_ESprite(lm_tile_c, 0, s_TD)
    else
      Draw2D_ESprite(lm_tile, 0, s_TD);

    with Tile[IX, IY] do
    begin
      if (Down)
      or (Alpha < 1) then
      begin
        if not EFree then
          Tile_NotFree_Draw
        else
          Tile_Free_Draw;
      end;

      if not Down then
      begin
        OpenGL.glColor4f(1, 1, 1, Alpha);
        Draw2D_ESprite(lm_tile_d, 0, s_TD);
      end;
    end;
    OpenGL.glColor4f(1, 1, 1, 1);
  end;
  //______________
begin
  glPushMatrix;
  glTranslated( StX - s_T - Round(Camera.X), StY - Round(Camera.Y), 0);
  for IY := 0 to H do
  begin
    for IX := 0 to W do
    begin
      glTranslated( s_T, 0, 0);
      if not Tile[IX, IY].nEnable then
        Tile_Draw;
    end;
  glTranslated( -(W + 1) * s_T, s_T, 0);
  end;
  glPopMatrix;

  for IX := 0 to Length(DTile.Element) - 1  do
  with DTile.Element[IX] do
  if Enabled then
    Draw2D_Sprite(lm_tile_d, pcX * s_T - Camera.X + StX, pcY * s_T - Camera.Y + StY, s_T, s_T, R, 0, Alpha);

  for IX := 0 to Length(Effect.Element) - 1  do
  with EFfect.Element[IX] do
  if Enabled then
    Draw2D_Sprite(lm_t_el[Tp], pcX * s_T - Camera.X + StX, pcY * s_T - Camera.Y + StY, s_T, s_T, 0, 0, Alpha, Size);
end;

end.
